from time import sleep
import unittest

from PyMD import MDGenerator
from pandas import DataFrame


class IndexingMethods(unittest.TestCase):
    mdGen = None

    def setup(self):
        self.mdGen = MDGenerator("example", title="Generated Markdown", author="Author")
        

    def test_single_bracketing(self):
        self.setup()
        self.mdGen["Section 1"].add_text("This is the first section.")
        self.mdGen["Section 2"].add_text("This is the second section.")
        self.mdGen["Section 2"].add_code("print('Hello, World!')")
        self.mdGen["Section 3"]["Subsection 1"].add_text("This is a subsection of the first section.")
        self.mdGen["Section 3"]["Subsection 2"].add_text("This is a subsection of the first section.a")
        self.mdGen["Section 3"]["Subsection 2"]["Subsubsection 1"].add_text("This is a subsubsection of the second subsection.")
        self.mdGen["Section 3"]["Subsection 2"]["Subsubsection 1"].add_text("This is a subsubsection of the second subsection.a")
        self.mdGen["Section 3"]["Subsection 2"]["Subsubsection 1"].add_text("This is a subsubsection of the second subsection.b")
        self.mdGen["Section 4"]["Subsection 1a"]["Subsubsection 1a"]["Subsubsubsection 1a"].add_text("This is a subsubsubsection of the second subsection.abc")

        self.mdGen.save()

        with open("example/GeneratedMD.md", "r") as f:
            data = f.read()
            self.assertIn("Section 1", data)
            self.assertIn("Section 2", data)
            self.assertIn("Section 3", data)
            self.assertIn("Subsection 1", data)
            self.assertIn("Subsection 2", data)
            self.assertIn("Subsection 1a", data)
            self.assertIn("Subsubsection 1", data)
            self.assertIn("Subsubsection 1a", data)
            self.assertIn("Subsubsubsection 1a", data)

            self.assertIn("This is the first section.", data)
            self.assertIn("This is the second section.", data)
            self.assertIn("print('Hello, World!')", data)
            self.assertIn("This is a subsection of the first section.", data)
            self.assertIn("This is a subsection of the first section.a", data)
            self.assertIn("This is a subsubsection of the second subsection.", data)
            self.assertIn("This is a subsubsection of the second subsection.a", data)
            self.assertIn("This is a subsubsection of the second subsection.b", data)
            self.assertIn("This is a subsubsubsection of the second subsection.abc", data)

    def test_multi_bracketing(self):
        self.setup()
        self.mdGen["Section 1/Subsection 1/Subsubsection 1"].add_text("This is a subsubsection of the first section.")
        self.mdGen["Section 1/Subsection 1/Subsubsection 1"].add_text("This is a subsubsection of the first section.a")
        self.mdGen["Section 2/Subsection 1a/Subsubsection 1a"].add_text("This is a subsubsection of the first section.b")
        self.mdGen["Section 2/Subsection 1a/Subsubsection 1a/meh"].add_text("This is a subsubsection of the first section.c")

        self.mdGen.save()

        with open("example/GeneratedMD.md", "r") as f:
            data = f.read()
            self.assertIn("Section 1", data)
            self.assertIn("Section 2", data)
            self.assertIn("Subsection 1", data)
            self.assertIn("Subsection 1a", data)
            self.assertIn("Subsubsection 1", data)
            self.assertIn("Subsubsection 1a", data)
            self.assertIn("meh", data)

            self.assertIn("This is a subsubsection of the first section.", data)
            self.assertIn("This is a subsubsection of the first section.a", data)
            self.assertIn("This is a subsubsection of the first section.b", data)
            self.assertIn("This is a subsubsection of the first section.c", data)

    def test_empty_subsections(self):
        self.setup()
        self.mdGen["Section 1//Subsubsection 1"].add_text("This is a subsubsection of the first section.")
        self.mdGen["Section 1//Subsubsection 1"].add_text("This is a subsubsection of the first section.a")
        self.mdGen["Section 2/Subsection 1a/"].add_text("This is a subsubsection of the first section.b")
        self.mdGen["/Subsection 1a/Subsubsection 1a"].add_text("This is a subsubsection of the first section.c")
        self.mdGen["Section 2/Subsection 1a///meh"].add_text("This is a subsubsection of the first section.d")
        self.mdGen["Section 3"][""][""].add_text("This is a subsubsection of the first section.e")

        self.mdGen.save()

        with open("example/GeneratedMD.md", "r") as f:
            data = f.read()
            self.assertIn("Section 1", data)
            self.assertIn("Section 2", data)
            self.assertIn("Section 3", data)
            self.assertIn("Subsection 1a", data)
            self.assertIn("Subsubsection 1", data)
            self.assertIn("Subsubsection 1a", data)
            self.assertIn("meh", data)

            self.assertIn("This is a subsubsection of the first section.", data)
            self.assertIn("This is a subsubsection of the first section.a", data)
            self.assertIn("This is a subsubsection of the first section.b", data)
            self.assertIn("This is a subsubsection of the first section.c", data)
            self.assertIn("This is a subsubsection of the first section.d", data)
            self.assertIn("This is a subsubsection of the first section.e", data)

    # def test_empty_base(self)


if __name__ == '__main__':
    unittest.main()


from time import sleep
import unittest

from pandas import DataFrame
from PyMD import MDGenerator
from matplotlib import pyplot as plt
import numpy as np


class AssignmentTypes(unittest.TestCase):
    mdGen = None

    def setup(self):
        self.mdGen = MDGenerator("example", title="Generated Markdown", author="Author")
        

    def test_string_assignment(self):
        self.setup()
        self.mdGen["Section 1"] = "This is the first section."
        self.mdGen.save()

        with open("example/GeneratedMD.md", "r") as f:
            data = f.read()
            self.assertIn("This is the first section.", data)

    def test_figure_assignment(self):
        self.setup()

        # Create a random figure
        fig, ax = plt.subplots()
        x = np.linspace(0, 2 * np.pi, 100)
        y = np.sin(x)
        ax.plot(x, y)
        self.mdGen["Section 1"] = fig
        self.mdGen.save()

        with open("example/GeneratedMD.md", "r") as f:
            data = f.read()
            self.assertIn("figures/GeneratedMD_image0.png", data)

    def test_dataframe_assignment(self):
        self.setup()

        # Create a random table
        headers = ["Header 1", "Header 2", "Header 3", "Header 4"]
        table = np.random.randint(0, 10, (6, 4))
        df = DataFrame(table, columns=headers)
        self.mdGen["Section 1"] = df
        self.mdGen.save()

        with open("example/GeneratedMD.md", "r") as f:
            data = f.read()
            self.assertIn("Header 1", data)
            self.assertIn("Header 2", data)
            self.assertIn("Header 3", data)
            self.assertIn("Header 4", data)

    def test_numpy_array_assignment(self):
        self.setup()

        # Create a random table
        table = np.random.randint(0, 10, (6, 4))
        self.mdGen["Section 1"] = table
        self.mdGen.save()

        with open("example/GeneratedMD.md", "r") as f:
            data = f.read()
            self.assertIn("Column 1", data)
            self.assertIn("Column 2", data)
            self.assertIn("Column 3", data)
            self.assertIn("Column 4", data)

    def test_list_assignment(self):
        self.setup()

        # Create a random list
        lst = ["Item 1", "Item 2", "Item 3"]
        self.mdGen["Section 1"] = lst
        self.mdGen.save()

        with open("example/GeneratedMD.md", "r") as f:
            data = f.read()
            self.assertIn("Item 1", data)
            self.assertIn("Item 2", data)
            self.assertIn("Item 3", data)


from abc import ABC, abstractmethod
from collections import UserDict
from enum import IntEnum
from pathlib import Path
from typing import Dict, List, Optional, Union
from matplotlib.figure import Figure
from mdutils.mdutils import MdUtils
import numpy as np
from pandas import DataFrame
from .utils import FIGURE_FOLDER


class SectionType(IntEnum):
    """
    Enum to define the type of section.
    """
    TEXT = 0
    CODE = 1
    IMAGE = 2
    TABLE = 3
    LIST = 4
    LINK = 5
    CHECKBOX = 6

    def new_dictionary(self) -> Dict:
        return {self.TEXT: 0, self.CODE: 0, self.IMAGE: 0, self.TABLE: 0, self.LIST: 0, self.LINK: 0, self.CHECKBOX: 0}

    def __str__(self):
        return self.name.lower()


class BaseSection(ABC):
    """
    Abstract class for a section in the markdown report. All types of sections must inherit from this class.
    """
    def __init__(self, mdFile: MdUtils, location:str):
        self.mdFile = mdFile
        self.location = location

    @abstractmethod
    def render(self, level:int=1, space_above:bool=False, space_below:bool=True):
        """
        Render the section in the markdown file. This section should include all the pre-processing required to render
        followed by a call to some MdUtils methods to render the section.

        Example MdUtils methods: new_header, new_paragraph, new_inline_image, insert_code, etc.
        """

    @abstractmethod
    def is_valid(self) -> bool:
        """
        Is the section valid to be rendered in the markdown file. If this returns False, the section will be ignored
        during rendering.
        """

class Section(BaseSection, UserDict):
    """
    Section to hold multiple sections in the markdown file.
    """
    save_path:Path = None
    dpi:int = None
    section_headers:List[str] = None
    section_type_count:Dict[SectionType,int] = None

    def __init__(self, mdFile: MdUtils, header:Optional[str] = None, location:Optional[str] = None):
        """
        Section to hold multiple sections in the markdown file.

        Args:
            mdFile (MdUtils): Markdown file object to render the sections.
            header (Optional[str], optional): Heading of the section. Defaults to None.
            location (Optional[str], optional): Location of the section. Defaults to None.
        """
        UserDict.__init__(self)
        BaseSection.__init__(self, mdFile, location)
        self.header = header
        if header is not None:
            self.section_headers.append(self.get_header_location())
        self.base_name = mdFile.file_name.split("/")[-1].split(".")[0]

        self.text:int = 0
        self.code:int = 0
        self.image:int = 0
        self.table:int = 0
        self.list:int = 0
        self.link:int = 0
        self.checkbox:int = 0
    
    def _from_json(self, json_dict:Dict) -> None:
        """
        Convert a dictionary to a markdown file.

        Args:
            json_dict (Dict): Dictionary representation of the markdown file.
        """
        location = self.get_header_location()
        relative_headers = [x.replace(location + "/", "") for x in self.section_headers if x.startswith(location + "/")]

        for key, value in json_dict.items():
            if key in relative_headers:
                self[key]._from_json(value)
            else:
                if isinstance(value, dict):
                    value_type = list(value.keys())[0]
                    if value_type == "text":
                        self.add_text(value["text"])
                    elif value_type == "code":
                        self.add_code(value["code"], value["language"])
                    elif value_type == "image_path":
                        self.add_image(value["image_path"], value["caption"])
                    elif value_type == "table":
                        self.add_table(value["table"], value["columns"], value["rows"])
                    elif value_type == "items":
                        self.add_list(value["items"])
                    elif value_type == "link":
                        self.add_link(value["link"], value["text"])
                    elif value_type == "text_list":
                        self.add_checkbox(value["text_list"], value["checked"])
                else:
                    raise ValueError(f"Value for key {key} is not a dictionary.")

    
    def _to_json(self) -> Dict:
        """
        Convert the markdown file to a normal dictionary.

        Returns:
            Dict: Dictionary representation of the markdown file.
        """
        json_dict = {}
        for key, value in self.items():
            json_dict[key] = value._to_json()
        return json_dict

    def get_section_ptr(self, keys:List[str]) -> BaseSection:
        ptr = self
        for k in keys[:-1]:
            ptr = ptr[k]
        return ptr, keys[-1]

    def __setitem__(self, key:str, section:Union[BaseSection, str, DataFrame, np.ndarray, Figure, List[str]]):
        if len(key) > 0 and key[0] == "/":
            key = key[1:]
        keys = key.split("/")
        
        if len(keys) > 1:
            ptr, new_key = self.get_section_ptr(keys)
            return ptr.__setitem__(new_key, section)
        else:
            if isinstance(section, BaseSection):
                return super().__setitem__(key, section)
            elif isinstance(section, str):
                self[key].add_text(section)
            elif isinstance(section, DataFrame):
                self[key].add_table(section)
            elif isinstance(section, np.ndarray):
                self[key].add_table(section)
            elif isinstance(section, Figure):
                self[key].add_image(section)
            elif isinstance(section, list):
                self[key].add_list(section)
            else:
                raise ValueError("Invalid section type.")
    
    def __getitem__(self, key:str) -> BaseSection:
        if len(key) > 0 and key[0] == "/":
            key = key[1:]
        keys = key.split("/")
        if len(keys) > 1:
            ptr, new_key = self.get_section_ptr(keys)
            return ptr.__getitem__(new_key)
        else:
            if key not in self:
                self[key] = Section(self.mdFile, key, self.get_header_location())
            return super().__getitem__(key)
    
    def __delitem__(self, key: str) -> None:     # Need to make this delete all strings with key in it
        if self.section_headers is not None and key in self.section_headers:
            self.section_headers.remove(key)
        return super().__delitem__(key)
    
    def get_header_location(self) -> str:
        """
        Get the heading of the section.

        Returns:
            str: Heading of the section.
        """
        return self.header if self.location == '' or self.location == None else self.location + '/' + self.header

    def get_section_name(self, section:BaseSection) -> str:
        """
        Get the name of the section based on the type of section.

        Args:
            section (BaseSection): Section object to get the name of.

        Returns:
            str: Name of the section.
        """
        if isinstance(section, Section):
            name = section.header
        elif isinstance(section, TextSection):
            name = f"text{self.text}"
        elif isinstance(section, CodeSection):
            name = f"code{self.code}"
        elif isinstance(section, ImageSection):
            name = f"image{self.image}"
        elif isinstance(section, TableSection):
            name = f"table{self.table}"
        elif isinstance(section, ListSection):
            name = f"list{self.list}"
        elif isinstance(section, LinkSection):
            name = f"link{self.link}"
        elif isinstance(section, CheckBoxSection):
            name = f"checkbox{self.checkbox}"
        else:
            raise ValueError("Invalid section type.")
        
        return name
    
    def update_section_count(self, section:BaseSection) -> None:
        """
        Update the section count based on the type of section.

        Args:
            section (BaseSection): Section object to update the count for.
        """
        if isinstance(section, TextSection):
            self.text += 1
            self.section_type_count[SectionType.TEXT] += 1
        elif isinstance(section, CodeSection):
            self.code += 1
            self.section_type_count[SectionType.CODE] += 1
        elif isinstance(section, ImageSection):
            self.image += 1
            self.section_type_count[SectionType.IMAGE] += 1
        elif isinstance(section, TableSection):
            self.table += 1
            self.section_type_count[SectionType.TABLE] += 1
        elif isinstance(section, ListSection):
            self.list += 1
            self.section_type_count[SectionType.LIST] += 1
        elif isinstance(section, LinkSection):
            self.link += 1
            self.section_type_count[SectionType.LINK] += 1
        elif isinstance(section, CheckBoxSection):
            self.checkbox += 1
            self.section_type_count[SectionType.CHECKBOX] += 1
        else:
            raise ValueError("Invalid section type.")
    
    def add_section(self, heading:Optional[str], section:Optional[BaseSection] = None) -> BaseSection:
        """
        Create a new section in the markdown file.

        Args:
            heading (str): Heading of the section.
            section (Optional[BaseSection], optional): Section to add to the markdown file. Defaults to None.
                - If None, a new Section object is created.

        Returns:
            bool: True if the section was added successfully, False if the section already exists.
        """
        if heading is None: # If the heading is None, add the section to the current section
            if section is None:
                return section
            else:
                self[self.get_section_name(section)] = section
                return section
        if len(heading) > 0 and heading[0] == "/":
            heading = heading[1:]
        headers = heading.split("/") # Split the heading into subheadings
        head = headers[0]

        if len(headers) == 1:  # If there is only one heading left, check if it is this section
            if head not in self.keys():  # If the section does not exist, create it
                self[head] = Section(self.mdFile, head, self.get_header_location())

            if section is not None:
                section = self[head].add_section(None, section)
            else:
                section = self[head]

        elif len(headers) > 1:
            if head not in self.keys():  # If the section does not exist, create it
                self[head] = Section(self.mdFile, head, self.get_header_location())

            section = self[head].add_section("/".join(headers[1:]), section)   # Recursively add the section to the next level
        
        return section
    
    def add_text(self, text:str="\n") -> BaseSection:
        """
        Add a text section to the markdown file.

        Args:
            heading (Optional[str], optional): Heading of the section to add to. Defaults to None.
                - If None, the text is added to the current section.
            text (str): Text to add to the markdown file.

        Returns:
            BaseSection: Text section object.
        """
        section = self.add_section(None, TextSection(self.mdFile, self.get_header_location(), text))
        if section is not None:
            self.text += 1
            self.section_type_count[SectionType.TEXT] += 1
        return section
    
    def add_code(self, code:str="\t", language:str="python") -> BaseSection:
        """
        Add a code section to the markdown file.

        Args:
            code (str): Code to add to the markdown file.
            language (str, optional): Language of the code. Defaults to "python".

        Returns:
            BaseSection: Code section object.
        """
        section = self.add_section(None, CodeSection(self.mdFile, self.get_header_location(), code, language))
        if section is not None:
            self.code += 1
            self.section_type_count[SectionType.CODE] += 1
        return section
    
    def add_figure(self, figure:Union[Figure,str], caption:str=None) -> BaseSection:
        """
        Add a figure section to the markdown file.

        Args:
            figure (Union[Figure,str]): Figure object or path to the figure file.
            caption (str, optional): Caption for the figure. Defaults to None.

        Returns:
            BaseSection: Image section object.
        """
        return self.add_image(figure, caption)
    
    def add_image(self, figure:Union[Figure,str], caption:str=None) -> BaseSection:
        """
        Add an image section to the markdown file.

        Args:
            image_path (str): Path to the image file.
            caption (str, optional): Caption for the image. Defaults to None.

        Returns:
            BaseSection: Image section object.
        """
        if isinstance(figure, Figure):
            # Save the figure
            image_path = self.save_path / FIGURE_FOLDER
            if not image_path.exists():
                image_path.mkdir()
            image_path = image_path / f"{self.base_name}_image{self.section_type_count[SectionType.IMAGE]}.png"
            figure.savefig(str(image_path), dpi=self.dpi)
            figure = str(image_path)

        section = self.add_section(None, ImageSection(self.mdFile, self.get_header_location(), figure, caption))
        if section is not None:
            self.image += 1
            self.section_type_count[SectionType.IMAGE] += 1
        return section
    
    def add_table(self, table:List[str], columns:int=3, rows:int=3) -> BaseSection:
        """
        Add a table section to the markdown file.

        Args:
            table (List[str]): Table data.
            columns (int, optional): Number of columns in the table. Defaults to 3.
            rows (int, optional): Number of rows in the table. Defaults to 3.

        Returns:
            BaseSection: Table section object.
        """
        if isinstance(table, DataFrame):
            rows, columns = table.shape
            col = table.columns.tolist()
            rows += 1
            table = col + [str(x) for x in table.values.flatten().tolist()]
        elif isinstance(table, np.ndarray):
            rows, columns = table.shape
            col = [f"Column {x+1}" for x in range(table.shape[1])]
            rows += 1
            table = col + [str(x) for x in table.flatten().tolist()]
        elif table is None:
            raise ValueError("Table cannot be None.")
            
        # Check if table size is valid
        if columns is not None and rows is not None:
            if columns * rows != len(table):
                raise ValueError("Table size does not match the number of rows and columns.")
        else:
            raise ValueError("Number of columns and rows must be specified.")

        section = self.add_section(None, TableSection(self.mdFile, self.get_header_location(), table, columns, rows))
        if section is not None:
            self.table += 1
            self.section_type_count[SectionType.TABLE] += 1
        return section
    
    def add_list(self, items:list, marked_with:str="-") -> BaseSection:
        """
        Add a list section to the markdown file.

        Args:
            items (list): List of items.

        Returns:
            BaseSection: List section object.
        """
        count = self.add_section(None, ListSection(self.mdFile, self.get_header_location(), items, marked_with))
        if count is not None:
            self.list += 1
            self.section_type_count[SectionType.LIST] += 1
        return count
    
    def add_link(self, link:str, text:Optional[str]) -> BaseSection:
        """
        Add a link section to the markdown file.

        Args:
            link (str): Link to add.
            text (str): Text to display for the link.

        Returns:
            BaseSection: Link section object.
        """
        section = self.add_section(None, LinkSection(self.mdFile, self.get_header_location(), link, text))
        if section is not None:
            self.link += 1
            self.section_type_count[SectionType.LINK] += 1
        return section
    
    def add_checkbox(self, text_list:List[str], checked:Union[List[bool],bool]=False) -> BaseSection:
        """
        Add a checkbox section to the markdown file.

        Args:
            text_list (List[str]): List of text items.
            checked (Union[List[bool],bool], optional): List of booleans indicating if the checkbox is checked. Defaults to False.

        Returns:
            BaseSection: Checkbox section object.
        """
        section = self.add_section(None, CheckBoxSection(self.mdFile, self.get_header_location(), text_list, checked))
        if section is not None:
            self.checkbox += 1
            self.section_type_count[SectionType.CHECKBOX] += 1
        return section
    
    def render(self, level:int=1, space_above:bool=False, space_below:bool=True):
        if space_above:
            self.mdFile.new_line()
        self.mdFile.new_header(level=level, title=self.header)
        if space_below:
            self.mdFile.new_line()

        level += 1
        for section_name, section in self.items():
            section.render(level=level)
    
    def is_valid(self) -> bool:
        for section_name, section in self:
            if not section.is_valid():
                return False
        return True
    

class TextSection(BaseSection):
    """
    Section to render text in the markdown file.
    """
    def __init__(self, mdFile: MdUtils, location:str, text:str):
        """
        Text section to render text in the markdown file.

        Args:
            mdFile (MdUtils): Markdown file object to render the text.
            location (str): Location of the text in the markdown file.
            text (str): Text to render in the markdown file.
        """
        super().__init__(mdFile, location)
        self.text = text

    def render(self, level:int=1, space_above:bool=False, space_below:bool=True):
        if space_above:
            self.mdFile.new_line()
        self.mdFile.new_paragraph(text=self.text)
        if space_below:
            self.mdFile.new_line()

    def is_valid(self) -> bool:
        return True
    
    def _to_json(self) -> Dict:
        return {
            "text": self.text
        }


class CodeSection(BaseSection):
    """
    Section to render code in the markdown file.
    """
    def __init__(self, mdFile: MdUtils, location:str, code:str, language:str="python"):
        """
        Code section to render code in the markdown file.

        Args:
            mdFile (MdUtils): Markdown file object to render the code.
            location (str): Location of the code in the markdown file.
            code (str): Code to render in the markdown file.
            language (str, optional): Language of the code. Defaults to "python".
        """
        super().__init__(mdFile, location)
        self.code = code
        self.language = language

    def render(self, level:int=1, space_above:bool=False, space_below:bool=True):
        if space_above:
            self.mdFile.new_line()
        self.mdFile.insert_code(code=self.code, language=self.language)
        if space_below:
            self.mdFile.new_line()

    def is_valid(self) -> bool:
        if len(self.code) == 0:
            return False
        return True
    
    def _to_json(self) -> Dict:
        return {
            "code": self.code,
            "language": self.language
        }
    

class ImageSection(BaseSection):
    """
    Section to render an image in the markdown file.
    """
    def __init__(self, mdFile: MdUtils, location:str, image_path:Union[str,Path], caption:Optional[str]=None):
        """
        Image section to render an image in the markdown file.

        Args:
            mdFile (MdUtils): Markdown file object to render the image.
            location (str): Location of the image in the markdown file.
            image_path (str): Path to the image file.
            caption (str, optional): Caption for the image. Defaults to None.
        """
        super().__init__(mdFile, location)
        self.image_path:Path = image_path if isinstance(image_path, Path) else Path(image_path)
        self.caption = caption if caption is not None else ""

    def render(self, level:int=1, space_above:bool=False, space_below:bool=True):
        if space_above:
            self.mdFile.new_line()
        image_path = self.image_path.resolve().relative_to(Path(self.mdFile.file_name).resolve().parent)
        self.mdFile.new_line(self.mdFile.new_inline_image(text=self.caption, path=str(image_path)))
        #self.mdFile.new_line(text=rf"![{self.caption}]({str(self.image_path)})")
        if space_below:
            self.mdFile.new_line()

    def is_valid(self) -> bool:
        return True
    
    def _to_json(self) -> Dict:
        return {
            "image_path": str(self.image_path),
            "caption": self.caption
        }
    

class TableSection(BaseSection):
    """
    Section to render a table in the markdown file.
    """
    def __init__(self, mdFile: MdUtils, location:str, table:List[str], columns:int=3, rows:int=3):
        """
        Table section to render a table in the markdown file.

        Args:
            mdFile (MdUtils): Markdown file object to render the table.
            location (str): Location of the table in the markdown file.
            table (List[str]): Table data.
            columns (int, optional): Number of columns in the table. Defaults to 3.
            rows (int, optional): Number of rows in the table. Defaults to 3.
        """
        super().__init__(mdFile, location)
        self.table = table
        self.columns = columns
        self.rows = rows

    def render(self, level:int=1, space_above:bool=False, space_below:bool=True):
        if space_above:
          self.mdFile.new_line()
        self.mdFile.new_table(columns=self.columns, rows=self.rows, text=self.table)
        if space_below:
          self.mdFile.new_line()

    def is_valid(self) -> bool:
        return True
    
    def _to_json(self) -> Dict:
        return {
            "table": self.table,
            "columns": self.columns,
            "rows": self.rows
        }

class ListSection(BaseSection):
    """
    Section to render a list in the markdown file.
    """
    def __init__(self, mdFile: MdUtils, location:str, items:list, marked_with:str="-"):
        """
        List section to render a list in the markdown file.

        Args:
            mdFile (MdUtils): Markdown file object to render the list.
            location (str): Location of the list in the markdown file.
            items (list): List of items.
        """
        super().__init__(mdFile, location)
        self.items = items
        self.marked_with = marked_with

    def render(self, level:int=1, space_above:bool=False, space_below:bool=True):
        if space_above:
            self.mdFile.new_line()
        self.mdFile.new_list(items=self.items, marked_with=self.marked_with)
        if space_below:
            self.mdFile.new_line()

    def is_valid(self) -> bool:
        return True
    
    def _to_json(self) -> Dict:
        return {
            "items": self.items
        }
    
class LinkSection(BaseSection):
    """
    Section to render a link in the markdown file.
    """
    def __init__(self, mdFile: MdUtils, location:str, link:str, text:Optional[str]):
        """
        Link section to render a link in the markdown file.

        Args:
            mdFile (MdUtils): Markdown file object to render the link.
            location (str): Location of the link in the markdown file.
            link (str): Link to add.
            text (str): Text to display for the link.
        """
        super().__init__(mdFile, location)
        self.link = link
        if text is None:
            self.text = link
        else:
            self.text = text

    def render(self, level:int=1, space_above:bool=False, space_below:bool=True):
        if space_above:
            self.mdFile.new_line()
        self.mdFile.new_line(text=rf"[{self.text}]({self.link})")
        if space_below:
            self.mdFile.new_line()

    def is_valid(self) -> bool:
        return True
    
    def _to_json(self) -> Dict:
        return {
            "link": self.link,
            "text": self.text
        }
    
class CheckBoxSection(BaseSection):
    """
    Section to render a checkbox in the markdown file.
    """
    def __init__(self, mdFile: MdUtils, location:str, text_list:List[str], checked:Union[List[bool],bool]=False):
        """
        Checkbox section to render a checkbox in the markdown file.

        Args:
            mdFile (MdUtils): Markdown file object to render the checkbox.
            location (str): Location of the checkbox in the markdown file.
            text_list (List[str]): List of text items.
            checked (Union[List[bool],bool], optional): List of booleans indicating if the checkbox is checked. Defaults to False.
        """
        super().__init__(mdFile, location)
        self.text_list = text_list
        if isinstance(checked, bool):
            self.checked = [checked for _ in range(len(text_list))]
        elif isinstance(checked, list) and (len(checked) == len(text_list)):
            self.checked = checked
        else:
            raise ValueError("checked must be a boolean or a list of booleans with the same length as text.")

    def render(self, level:int=1, space_above:bool=False, space_below:bool=True):
        if space_above:
            self.mdFile.new_line()
        for text, checked in zip(self.text_list, self.checked):
            self.mdFile.new_line(text=rf"- [x]" + f" {text}" if checked else rf"- [ ]" + f" {text}")
        if space_below:
            self.mdFile.new_line()

    def is_valid(self) -> bool:
        return True
    
    def _to_json(self) -> Dict:
        return {
            "text_list": self.text_list,
            "checked": self.checked
        }



from typing import List

FILE_TYPES = [".md", ".markdown"]
FIGURE_FOLDER = "figures"

def is_file_type(file_name: str, file_types: List[str] = FILE_TYPES) -> bool:
    """
    Check if the file is of the given type

    Args:
        file_name (str): Name of the file
        file_types (list[str]): List of file types

    Returns:
        bool: True if the file is of the given type
    """
    for file_type in file_types:
        if file_name.endswith(file_type):
            return True
    return False


"""
To do:
    - Add subsections to list
    - Add section links/references
"""
import json
from matplotlib.figure import Figure
from mdutils.mdutils import MdUtils
from pathlib import Path
from typing import Dict, Optional, Union, List
from .tools.utils import is_file_type, FIGURE_FOLDER
from .tools.sections import *


class MDGenerator(Section):
    """
    Markdown Generator class to generate markdown files.
    """
    def __init__(self, save_path: Optional[Union[str,Path]] = None, file_name:str = "GeneratedMD", title:Optional[str] = None, author:Optional[Union[str,List[str]]] = None, dpi:Optional[int] = None):
        """
        Initialize the markdown generator.

        Args:
            save_path (Optional[Union[str,Path]], optional): Path to save the generated file. If None, the current working directory is used.
            file_name (str, optional): Name of the generated file. Defaults to "GeneratedMD".
            title (str, optional): Title of the document. Defaults to None.
            author (str, optional): Author of the document. Defaults to None.
            dpi (int, optional): DPI of the generated image. Defaults to None, figures dpi.
        """
        if save_path is None:
            self.save_path = Path.cwd()
        elif isinstance(save_path, str):
            if is_file_type(save_path):
                file_name = ".".join(save_path.split("/")[-1].split(".")[:-1])
                save_path = "/".join(save_path.split("/")[:-1])
            self.save_path = Path(save_path)
        elif isinstance(save_path, Path):
            self.save_path = save_path
        else:
            raise TypeError("save_path must be a str or Path object")

        self.save_path.mkdir(parents=True, exist_ok=True)

        self.file_name = file_name
        if title is None:
            self.title = ""
        else:
            self.title = title
        if author is None:
            self.author = ""
        else:
            if isinstance(author, list):
                self.author = "Authors: " + ", ".join(author)
            else:
                self.author = f"Author: {author}"
        
        self.dpi = dpi  # Store dpi for the figures
        
        mdFile = MdUtils(file_name=str(self.save_path / (file_name + ".md")), title=self.title, author=self.author)
        self.section_headers:List[str] = []
        self.section_type_count:Dict[SectionType,int] = SectionType.new_dictionary(SectionType)
        
        # Set the class variables for the Section class
        Section.save_path = self.save_path
        Section.dpi = self.dpi
        Section.section_headers = self.section_headers
        Section.section_type_count = self.section_type_count

        # Initialize the Parent Section class
        Section.__init__(self, mdFile)

    def section_search(self, key:str) -> Section:
        """
        Search for the section with the given key.
        
        Args:
            key (str): Key to search for.

        Returns:
            Optional[BaseSection]: Section with the given key.
        """
        if len(key) > 0 and key[0] == "/":
            key = key[1:]
        keys = key.split("/")
        
        ptr = self
        for k in keys:
            ptr = ptr[k]
        return ptr
            
    def add_text(self, header:Optional[str]=None, text:str="\n") -> BaseSection:
        """
        Add a text section to the markdown file.

        Args:
            header (str): Header of the section.
            text (str): Text of the section.
        """
        if header is None:
            super().add_text(text)
        else:
            section:Section = self.section_search(header)
            section.add_text(text)

    def add_code(self, header:Optional[str]=None, code:str="\n") -> BaseSection:
        """
        Add a code section to the markdown file.

        Args:
            header (str): Header of the section.
            code (str): Code of the section.
        """
        if header is None:
            super().add_code(code)
        else:
            section:Section = self.section_search(header)
            section.add_code(code)

    def add_image(self, header:Optional[str]=None, fig:Union[Figure,str]="", caption:str="") -> BaseSection:
        """
        Add an image section to the markdown file.

        Args:
            header (str): Header of the section.
            fig (Figure): Figure to add to the section.
            caption (str): Caption of the section.
        """
        if header is None:
            super().add_image(fig, caption)
        else:
            section:Section = self.section_search(header)
            section.add_image(fig, caption)

    def add_list(self, header:Optional[str]=None, items:List[str]=[], marked_with:str="-") -> BaseSection:
        """
        Add a list section to the markdown file.

        Args:
            header (str): Header of the section.
            items (List[str]): Items of the section.
        """
        if header is None:
            super().add_list(items, marked_with)
        else:
            section:Section = self.section_search(header)
            section.add_list(items, marked_with)

    def add_link(self, header:Optional[str]=None, link:str="", text:str="") -> BaseSection:
        """
        Add a link section to the markdown file.

        Args:
            header (str): Header of the section.
            link (str): Link of the section.
            text (str): Text of the section.
        """
        if header is None:
            super().add_link(link, text)
        else:
            section:Section = self.section_search(header)
            section.add_link(link, text)

    def add_checkbox(self, header:Optional[str]=None, items:List[str]=[], checked:List[bool]=[]) -> BaseSection:
        """
        Add a checkbox section to the markdown file.

        Args:
            header (str): Header of the section.
            items (List[str]): Items of the section.
            checked (List[bool]): Checked status of the items.
        """
        if header is None:
            super().add_checkbox(items, checked)
        else:
            section:Section = self.section_search(header)
            section.add_checkbox(items, checked)

    def render(self) -> None:
        """
        Render the markdown file.
        """
        for section in self.values():
            section.render()
        self.mdFile.create_md_file()

    def is_valid(self) -> bool:
        """
        Check if the markdown file is valid.

        Returns:
            bool: True if the markdown file is valid, False otherwise.
        """
        for section in self.values():
            if not section.is_valid():
                return False
        return True
    
    def save(self, file_name:Optional[str] = None) -> None:
        """
        Save the markdown file.

        Args:
            file_name (Optional[str], optional): Name of the file to save. Defaults to None.
                - If None, the current file name is used.
        """
        if file_name is not None:
            if len(file_name.split("/")) > 1:
                self.save_path = Path("/".join(file_name.split("/")[:-1]))
                self.file_name = file_name.split("/")[-1]
            if file_name.split(".")[-1] == "md":
                self.file_name = ".".join(file_name.split(".")[:-1])

            self.mdFile.file_name = str(self.save_path / (self.file_name + ".md"))

        self.render()

    def _to_json(self) -> Dict:
        """
        Convert the markdown file to a normal dictionary.

        Returns:
            Dict: Dictionary representation of the markdown file.
        """
        json_dict = {}

        # Add settings
        json_dict["MDG_Settings"] = {
            "save_path": str(self.save_path),
            "file_name": self.file_name,
            "title": self.title,
            "author": self.author,
            "dpi": self.dpi,
            "section_headers": self.section_headers,
            "section_type_count": self.section_type_count
        }

        for key, value in self.items():
            json_dict[key] = value._to_json()
        return json_dict

    def save_json(self, file_name:Optional[str] = None) -> None:
        """
        Save the markdown file as a json file.

        Args:
            file_name (Optional[str], optional): Name of the json file. Defaults to None.
                - If None, the file is saved as "GeneratedMD.json".
        """
        if file_name is not None:
            if len(file_name.split("/")) > 1:
                save_path = Path("/".join(file_name.split("/")[:-1]))
                file_name = file_name.split("/")[-1]
            else:
                save_path = self.save_path

            if file_name.split(".")[-1] == "json":
                file_name = ".".join(file_name.split(".")[:-1])
        else:
            save_path = self.save_path
            file_name = self.file_name

        with open(str(save_path / (file_name + ".json")), "w") as f:
            json.dump(self._to_json(), f, indent=4)

    def load_json(self, file_name:Optional[str] = None) -> None:
        """
        Load a json file to create a markdown file.

        Args:
            file_name (Optional[str], optional): Name of the json file. Defaults to None.
                - If None, the file is loaded as "GeneratedMD.json".
        """
        if file_name is not None:
            if len(file_name.split("/")) > 1:
                save_path = Path("/".join(file_name.split("/")[:-1]))
                file_name = file_name.split("/")[-1]
            else:
                save_path = self.save_path

            if file_name.split(".")[-1] == "json":
                file_name = ".".join(file_name.split(".")[:-1])
        else:
            save_path = self.save_path
            file_name = self.file_name

        with open(str(save_path / (file_name + ".json")), "r") as f:
            json_dict = json.load(f)
        
        self.save_path = Path(json_dict["MDG_Settings"]["save_path"])
        self.file_name = json_dict["MDG_Settings"]["file_name"]
        self.title = json_dict["MDG_Settings"]["title"]
        self.author = json_dict["MDG_Settings"]["author"]
        self.dpi = json_dict["MDG_Settings"]["dpi"]
        self.section_headers = json_dict["MDG_Settings"]["section_headers"]
        self.section_type_count = SectionType.new_dictionary(SectionType)

        # Set the class variables for the Section class
        Section.save_path = self.save_path
        Section.dpi = self.dpi
        Section.section_headers = self.section_headers
        Section.section_type_count = self.section_type_count

        self.mdFile = MdUtils(file_name=str(self.save_path / (self.file_name + ".md")), title=self.title, author=self.author)
        for key, value in json_dict.items():
            if key != "MDG_Settings":
                if key in self.section_headers: # It is a Section Object
                    self[key]._from_json(value)
                else:       # It is not a Section Object
                    if isinstance(value, dict):
                        value_type = list(value.keys())[0]
                        if value_type == "text":
                            self.add_text(None, value["text"])
                        elif value_type == "code":
                            self.add_code(None, value["code"], value["language"])
                        elif value_type == "image":
                            self.add_image(None, value["image"], value["caption"])
                        elif value_type == "table":
                            self.add_table(None, value["table"], value["columns"], value["rows"])
                        elif value_type == "list":
                            self.add_list(None, value["list"])
                        elif value_type == "link":
                            self.add_link(None, value["link"], value["text"])
                        elif value_type == "checkbox":
                            self.add_checkbox(None, value["text_list"], value["checked"])
                    else:
                        raise ValueError(f"Value for key {key} is not a dictionary.")
        
        # Remove duplicate section headers
        i = 0
        while i < len(self.section_headers):
            if self.section_headers[i] in self.section_headers[:i]:
                self.section_headers.pop(i)
            else:
                i += 1



# if __name__ == "__main__":
#     # mdGen = MDGenerator()
#     # mdGen.load_json("example/GeneratedMD.json")
#     # mdGen.save()

#     mdGen = MDGenerator("example", title="Generated Markdown", author="Author")
#     mdGen.add_text("Section 1", "This is the first section.")
#     #mdGen["Section 1"].add_text("This is a subsection of the first section.")
#     # mdGen.add_text("Section 2", "This is the second section.")
#     mdGen["Section 2"].add_text("This is a subsection of the second section.")
#     # mdGen.add_code("Section 2", "print('Hello, World!')")
#     mdGen["Section 2"].add_code("print('This is a subsection of the second section.')")
#     # mdGen.add_text("Section 1/Subsection 1", "This is a subsection of the first section.")
#     mdGen["Section 1"]["Subsection 1"].add_text("This is a subsection of the first section.")
#     # mdGen.add_text("Section 1/Subsection 2", "This is a subsection of the first section.")
#     mdGen["Section 1"]["Subsection 2"].add_text("This is a subsection of the first section.")
#     mdGen["Section 1/Subsection 2"].add_text("Specifically, this is the second subsection of the first subsection.")

#     # mdGen.add_text("Section 1/Subsection 2/Subsubsection 1", "This is a subsubsection of the second subsection.")
#     mdGen["Section 1"]["Subsection 2"]["Subsubsection 1"].add_text("This is a subsubsection of the second subsection.")

#     mdGen["Section 1"] = "Hello"

#     import matplotlib.pyplot as plt
#     import numpy as np

#     # Generate random data
#     x = np.linspace(0, 10, 100)
#     y = np.random.randn(100)

#     # Create a figure and plot the data
#     fig, ax = plt.subplots()
#     ax.plot(x, y)

#     # Add labels and title
#     ax.set_xlabel('X-axis')
#     ax.set_ylabel('Y-axis')
#     ax.set_title('Random Figure')

#     # Add the image to the markdown file
#     # mdGen.add_image("Section 1", fig, "This is a random figure.")
#     mdGen["Section 1"].add_image(fig, "This is a random figure.")
#     mdGen["Section 1"].add_image(fig, "This is a random figure.")
#     mdGen["Section 1"].add_image(fig, "This is a random figure.")

#     # Add a list to the markdown file
#     # mdGen.add_list("Section 2", ["Item 1", "Item 2", "Item 3"])
#     mdGen["Section 2"].add_list(["Item 1", "Item 2", "Item 3"])

#     # Add a link to the markdown file
#     # mdGen.add_link("Section 2", "https://www.google.com", "Google")
#     mdGen["Section 2"].add_link("https://www.google.com", "Google")

#     # Add a checkbox list to the markdown file
#     # mdGen.add_checkbox("Section 2", ["Check 1", "Check 2", "Check 3"], [True, False, True])
#     mdGen["Section 2"].add_checkbox(["Check 1", "Check 2", "Check 3"], [True, False, True])

#     # Add a table to the markdown file
#     headers = ["Header 1", "Header 2", "Header 3", "Header 4"]
#     table = np.random.randint(0, 10, (6, 4))
#     df = DataFrame(table, columns=headers)

#     list_table = ["Header 1", "Header 2", "Header 3"] + [str(3*i + j) for i in range(3) for j in range(3)]
#     # mdGen.add_table("Section 3/Numpy Array Table", table)
#     mdGen["Section 3"]["Numpy Array Table"].add_table(table)
#     # mdGen.add_table("Section 3/Python List Table", list_table, 3, 4)
#     mdGen["Section 3"]["Python List Table"].add_table(list_table, 3, 4)
#     # mdGen.add_table("Section 3/Pandas DataFrame Table", df)
#     mdGen["Section 3"]["Pandas DataFrame Table"].add_table(df)

#     # Save the markdown file
#     mdGen.save_json()
#     mdGen.save()


import unittest

from pandas import DataFrame
from PyMD import MDGenerator


class SavingFiles(unittest.TestCase):
    mdGen = None

    def setup(self):
        if self.mdGen is not None:
            return
        
        mdGen = MDGenerator("/home/rlfowler/Documents/myprojects/PyMD/example", file_name='GeneratedMD', title="Generated Markdown", author="Author")
        mdGen.add_text("Section 1", "This is the first section.")
        mdGen.add_text("Section 2", "This is the second section.")
        mdGen.add_code("Section 2", "print('Hello, World!')")
        mdGen.add_text("Section 1/Subsection 1", "This is a subsection of the first section.")
        mdGen.add_text("Section 1/Subsection 2", "This is a subsection of the first section.")
        mdGen.add_text("Section 1/Subsection 2/Subsubsection 1", "This is a subsubsection of the second subsection.")

        import matplotlib.pyplot as plt
        import numpy as np

        # Generate random data
        x = np.linspace(0, 10, 100)
        y = np.random.randn(100)

        # Create a figure and plot the data
        fig, ax = plt.subplots()
        ax.plot(x, y)

        # Add labels and title
        ax.set_xlabel('X-axis')
        ax.set_ylabel('Y-axis')
        ax.set_title('Random Figure')

        # Add the image to the markdown file
        mdGen.add_image("Section 1", fig, "This is a random figure.")

        # Add a list to the markdown file
        mdGen.add_list("Section 2", ["Item 1", "Item 2", "Item 3"])

        # Add a link to the markdown file
        mdGen.add_link("Section 2", "https://www.google.com", "Google")

        # Add a checkbox list to the markdown file
        mdGen.add_checkbox("Section 2", ["Check 1", "Check 2", "Check 3"], [True, False, True])


        # Add a table to the markdown file
        headers = ["Header 1", "Header 2", "Header 3", "Header 4"]
        table = np.random.randint(0, 10, (6, 4))
        df = DataFrame(table, columns=headers)
        
        list_table = ["Header 1", "Header 2", "Header 3"] + [str(3*i + j) for i in range(3) for j in range(3)]
        # mdGen.add_table("Section 3/Numpy Array Table", table)
        mdGen["Section 3"]["Numpy Array Table"].add_table(table)
        # mdGen.add_table("Section 3/Python List Table", list_table, 3, 4)
        mdGen["Section 3"]["Python List Table"].add_table(list_table, 3, 4)
        # mdGen.add_table("Section 3/Pandas DataFrame Table", df)
        mdGen["Section 3"]["Pandas DataFrame Table"].add_table(df)

        # Save the markdown file
        self.mdGen = mdGen

    def test_json(self):
        self.setup()
        self.mdGen.save_json()

        with open("example/GeneratedMD.json", "r") as f:
            data = f.read()
            self.assertIn("Section 1", data)
            self.assertIn("Section 2", data)
            self.assertIn("Section 3", data)
            self.assertIn("Subsection 1", data)
            self.assertIn("Subsection 2", data)
            self.assertIn("Subsubsection 1", data)
            self.assertIn("This is the first section.", data)
            self.assertIn("This is the second section.", data)
            self.assertIn("print('Hello, World!')", data)
            self.assertIn("This is a subsection of the first section.", data)
            self.assertIn("This is a subsection of the first section.", data)
            self.assertIn("This is a subsubsection of the second subsection.", data)
            self.assertIn("This is a random figure.", data)
            self.assertIn("Item 1", data)
            self.assertIn("Item 2", data)
            self.assertIn("Item 3", data)
            self.assertIn("https://www.google.com", data)
            self.assertIn("Google", data)
            self.assertIn("Check 1", data)
            self.assertIn("Check 2", data)
            self.assertIn("Check 3", data)
            self.assertIn("Header 1", data)
            self.assertIn("Header 2", data)
            self.assertIn("Header 3", data)
            self.assertIn("Header 4", data)
            self.assertIn("Numpy Array Table", data)
            self.assertIn("Python List Table", data)
            self.assertIn("Pandas DataFrame Table", data)
            self.assertIn("This is a random figure.", data)
            self.assertIn("This is a subsection of the first section.", data)
            self.assertIn("This is a subsection of the first section.", data)
            self.assertIn("This is a subsubsection of the second subsection.", data)
            self.assertIn("This is a random figure.", data)
            self.assertIn("Item 1", data)
            self.assertIn("Item 2", data)
            self.assertIn("Item 3", data)
            self.assertIn("https://www.google.com", data)
            self.assertIn("Google", data)
            self.assertIn("Check 1", data)
            self.assertIn("Check 2", data)
            self.assertIn("Check 3", data)
            self.assertIn("Header 1", data)
            self.assertIn("Header 2", data)

    def test_load_json(self):
        self.setup()
        self.mdGen.save_json()
        mdGen = MDGenerator("/home/rlfowler/Documents/myprojects/PyMD/example", file_name='GeneratedMD', title="Generated Markdown", author="Author")
        mdGen.load_json()

        self.assertListEqual(self.mdGen.section_headers, mdGen.section_headers)
        self.assertDictEqual(self.mdGen.section_type_count, mdGen.section_type_count)


if __name__ == "__main__":
    unittest.main()
            


from .MDGenerator import MDGenerator
#from tools.sections import Section, TextSection, CodeSection, ListSection, TableSection, ImageSection, LinkSection, CheckBoxSection

__all__ = ["MDGenerator"]#, "Section", "TextSection", "CodeSection", "ListSection", "TableSection", "ImageSection", "LinkSection", "CheckBoxSection"]


from time import sleep
import unittest

from pandas import DataFrame
from PyMD import MDGenerator


class AddingRandomStuff(unittest.TestCase):
    setup_done = False

    def setup(self):
        if self.setup_done:
            return
        
        mdGen = MDGenerator("/home/rlfowler/Documents/myprojects/PyMD/example", file_name='GeneratedMD', title="Generated Markdown", author="Author")
        mdGen.add_text("Section 1", "This is the first section.")
        mdGen.add_text("Section 2", "This is the second section.")
        mdGen.add_code("Section 2", "print('Hello, World!')")
        mdGen.add_text("Section 1/Subsection 1", "This is a subsection of the first section.")
        mdGen.add_text("Section 1/Subsection 2", "This is a subsection of the first section.")
        mdGen.add_text("Section 1/Subsection 2/Subsubsection 1", "This is a subsubsection of the second subsection.")

        import matplotlib.pyplot as plt
        import numpy as np

        # Generate random data
        x = np.linspace(0, 10, 100)
        y = np.random.randn(100)

        # Create a figure and plot the data
        fig, ax = plt.subplots()
        ax.plot(x, y)

        # Add labels and title
        ax.set_xlabel('X-axis')
        ax.set_ylabel('Y-axis')
        ax.set_title('Random Figure')

        # Add the image to the markdown file
        mdGen.add_image("Section 1", fig, "This is a random figure.")

        # Add a list to the markdown file
        mdGen.add_list("Section 2", ["Item 1", "Item 2", "Item 3"])

        # Add a link to the markdown file
        mdGen.add_link("Section 2", "https://www.google.com", "Google")

        # Add a checkbox list to the markdown file
        mdGen.add_checkbox("Section 2", ["Check 1", "Check 2", "Check 3"], [True, False, True])


        # Add a table to the markdown file
        headers = ["Header 1", "Header 2", "Header 3", "Header 4"]
        table = np.random.randint(0, 10, (6, 4))
        df = DataFrame(table, columns=headers)
        
        list_table = ["Header 1", "Header 2", "Header 3"] + [str(3*i + j) for i in range(3) for j in range(3)]
        # mdGen.add_table("Section 3/Numpy Array Table", table)
        mdGen["Section 3"]["Numpy Array Table"].add_table(table)
        # mdGen.add_table("Section 3/Python List Table", list_table, 3, 4)
        mdGen["Section 3"]["Python List Table"].add_table(list_table, 3, 4)
        # mdGen.add_table("Section 3/Pandas DataFrame Table", df)
        mdGen["Section 3"]["Pandas DataFrame Table"].add_table(df)

        # Save the markdown file
        mdGen.save()
        self.setup_done = True

    def test_add_text(self):
        self.setup()
        with open("/home/rlfowler/Documents/myprojects/PyMD/example/GeneratedMD.md", "r") as file:
            data = file.read()
            self.assertIn("This is the first section.", data)
            self.assertIn("This is the second section.", data)
            self.assertIn("This is a subsection of the first section.", data)
            self.assertIn("This is a subsection of the first section.", data)
            self.assertIn("This is a subsubsection of the second subsection.", data)

    def test_add_code(self):
        self.setup()
        with open("/home/rlfowler/Documents/myprojects/PyMD/example/GeneratedMD.md", "r") as file:
            data = file.read()
            self.assertIn("```python\nprint('Hello, World!')\n```", data)

    def test_add_image(self):
        self.setup()
        with open("/home/rlfowler/Documents/myprojects/PyMD/example/GeneratedMD.md", "r") as file:
            data = file.read()
            self.assertIn("This is a random figure.", data)

    def test_add_list(self):
        self.setup()
        with open("/home/rlfowler/Documents/myprojects/PyMD/example/GeneratedMD.md", "r") as file:
            data = file.read()
            self.assertIn("- Item 1\n- Item 2\n- Item 3", data)

    def test_add_link(self):
        self.setup()
        with open("/home/rlfowler/Documents/myprojects/PyMD/example/GeneratedMD.md", "r") as file:
            data = file.read()
            self.assertIn("[Google](https://www.google.com)", data)

    def test_add_checkbox(self):
        self.setup()
        with open("/home/rlfowler/Documents/myprojects/PyMD/example/GeneratedMD.md", "r") as file:
            data = file.read()
            self.assertIn("- [x] Check 1  \n- [ ] Check 2  \n- [x] Check 3  ", data)

    # def test_add_table(self):
    #     self.setup()
    #     with open("/home/rlfowler/Documents/myprojects/PyMD/example/GeneratedMD.md", "r") as file:
    #         data = file.read()
    #         self.assertIn("|Header 1|Header 2|Header 3|Header 4|\n|---|---|---|---|\n|0|3|2|4|\n|9|0|5|2|\n|3|9|7|6|\n|9|3|5|5|\n|0|5|2|5|\n|4|6|3|9|", data)
    #         self.assertIn("|Header 1|Header 2|Header 3|Header 4|\n|---|---|---|---|\n|Header 1|Header 2|Header 3|0|\n|1|2|3|3|\n|4|5|6|4|\n|7|8|0|5|", data)
    #         self.assertIn("|Header 1|Header 2|Header 3|Header 4|\n|---|---|---|---|\n|0|1|2|3|\n|4|5|6|7|\n|8|9|0|1|\n|2|3|4|5|\n|6|7|8|9|", data)
            
    # def test_add_multiple_images(self):



if __name__ == '__main__':
    unittest.main()